package com.leromro.core.controller;

import java.util.List;
import java.util.concurrent.TimeUnit;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.binarywang.wxpay.bean.notify.WxPayNotifyResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.leromro.common.core.domain.R;
import com.leromro.common.core.page.TableDataInfo;
import com.leromro.common.utils.SecurityUtils;
import com.leromro.core.domain.Address;
import com.leromro.core.domain.MainOrders;
import com.leromro.core.domain.SecondOrder;
import com.leromro.core.domain.VolunteerInfo;
import com.leromro.core.domain.dto.OrderRequestDTO;
import com.leromro.core.domain.dto.UserOrderReviewDTO;
import com.leromro.core.domain.vo.*;
import com.leromro.core.facade.OrdersFacade;
import com.leromro.core.service.IAddressService;
import com.leromro.core.service.IMainOrderService;
import com.leromro.core.service.ISecondOrderService;
import com.leromro.core.service.IVolunteerInfoService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.leromro.common.annotation.Log;
import com.leromro.common.core.controller.BaseController;
import com.leromro.common.core.domain.AjaxResult;
import com.leromro.common.enums.BusinessType;

/**
 * 订单主Controller
 *
 * @author ruoyi
 * @date 2025-04-09
 */
@Slf4j
@RestController
@Api(tags = "用户订单")
@RequestMapping("/core/users/orders")
public class OrdersController extends BaseController {
    @Autowired
    private IMainOrderService mainOrderService;
    @Autowired
    private ISecondOrderService secondOrderService;
    @Autowired
    private IAddressService addressService;
    @Autowired
    private IVolunteerInfoService volunteerInfoService;
    @Autowired
    private OrdersFacade ordersFacade;


    /**
     * 后台管理web页面订单列表
     */
    @ApiOperation("后台管理web页面订单列表")
    @GetMapping("/web/mainOrderList")
    public TableDataInfo<webMainOrderListVO> webMainOrderList(MainOrdersVO mainOrdersVO) {
        startPage();
        List<webMainOrderListVO> list = mainOrderService.webMainOrderList(mainOrdersVO);
        return getDataTable(list);
    }



    /**
     * 查询订单主列表
     */
    @ApiOperation("用户查询订单列表")
    @GetMapping("/mainOrderList")
    public TableDataInfo<MainOrdersVO> selectOrderList(String[] orderStatus) {
        //startPage();
        Long userId = SecurityUtils.getUserId();
        List<MainOrdersVO> list = mainOrderService.selectByUserId(orderStatus, userId);
        return getDataTable(list);
    }



    /**
     * 获取订单主详细信息
     */
    @ApiOperation("用户获取订单详细信息以及大订单所属小订单")
    @GetMapping("orderInfo/{mainOrderId}")
    public R<OrdersInfoVO> getMainOrderInfo(@PathVariable("mainOrderId") Long mainOrderId) {
        QueryWrapper<MainOrders> wrapper = new QueryWrapper<>();
        Long userId = SecurityUtils.getUserId();
        wrapper.eq("main_order_id", mainOrderId);
        MainOrders mainOrders = mainOrderService.list(wrapper).get(0);
        QueryWrapper<SecondOrder> wrapper1 = new QueryWrapper<>();
        wrapper1.eq("main_order_id", mainOrderId);
        List<SecondOrder> list = secondOrderService.list(wrapper1);
        //地址改版，所有地址都在订单中获取
        Address address = Address.builder().address(mainOrders.getAddress()).name(mainOrders.getClientName()).telephone(mainOrders.getClientTelephone()).build();
        /* QueryWrapper<Address> addressQueryWrapper = new QueryWrapper<Address>().eq("address_id", mainOrders.getAddressId());
        Address address = addressService.getOne(addressQueryWrapper);*/
        QueryWrapper<VolunteerInfo> volunteerInfoQueryWrapper = new QueryWrapper<VolunteerInfo>().eq("volunteer_id", mainOrders.getVolunteerId()).eq("service_category",mainOrders.getServiceCategory());
        VolunteerInfo volunteerInfo = volunteerInfoService.list(volunteerInfoQueryWrapper).get(0);
        OrdersInfoVO ordersInfoVO = new OrdersInfoVO();
        ordersInfoVO.setMainOrders(mainOrders);
        ordersInfoVO.setSecondOrderList(list);
        ordersInfoVO.setAddress(address);
        ordersInfoVO.setVolunteerInfo(volunteerInfo);
        return R.ok(ordersInfoVO);
    }

    /**
     * 新增订单主
     */
    @ApiOperation("用户创建订单")
    @Log(title = "创建订单", businessType = BusinessType.INSERT)
    @PostMapping("/createOrder")
    public R<OrderPaymentVO> createOrder(@RequestBody OrderRequestDTO orderRequest) {

        return mainOrderService.createOrders(orderRequest);
    }

    /**
     * 用户点击完成订单 , 根据评价算钱。
     */
    @ApiOperation(value = "用户评价小订单",notes = "传入对象参数 包含： secondOrderId, 四个score, userReview, userPicture")
    @PostMapping("/userFinishOrder")
    public AjaxResult userFinish (@RequestBody UserOrderReviewDTO reviewDTO) {

        secondOrderService.userFinish(reviewDTO);
        return AjaxResult.success();
    }


    /**
     * 管理端 订单列表查询
     */
    @ApiOperation(value = "web管理端订单详情信息查询",notes = "传入订单mainOrderId")
    @GetMapping("/web/orderInfo/{mainOrderId}")
    public R<WebMainOrdersInfoVO> webOrderInfo (@PathVariable Long mainOrderId) {
        WebMainOrdersInfoVO webMainOrderInfoVO = mainOrderService.selectWebMainOrderInfoVO(mainOrderId);
        List<SecondOrder> secondOrderList = secondOrderService.list(new LambdaQueryWrapper<SecondOrder>().eq(SecondOrder::getMainOrderId, mainOrderId));
        webMainOrderInfoVO.setSecondOrderList(secondOrderList);
        return R.ok(webMainOrderInfoVO);
    }
/*
    */
/**
     * 后续拆分下单和支付接口
     *//*

    @ApiOperation("用户支付订单")
    @Log(title = "支付订单", businessType = BusinessType.INSERT)
    @PostMapping("/createOrder")
    public AjaxResult PayOrder(@RequestBody OrderRequestDTO orderRequest) {

        return AjaxResult.success();
    }
*/




    /**
     * 支付超时或取消
     */
    public AjaxResult PayCancel(MainOrders order) {
        ordersFacade.PayCancel(order);
        return AjaxResult.success();
    }

    /**
     * 订单支付回调
     */
    @PostMapping("/payNotify")
    public String payNotify(@RequestBody String xmlData) {
        return ordersFacade.payNotify(xmlData);
    }






}
