package com.leromro.core.facade;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.binarywang.wxpay.bean.notify.WxPayNotifyResponse;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.service.WxPayService;
import com.leromro.core.domain.MainOrders;
import com.leromro.core.service.IMainOrderService;
import com.leromro.core.service.IOrderRefundService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;

import java.util.concurrent.TimeUnit;

@Slf4j
@Service
public class OrdersFacade {

    @Autowired
    private IMainOrderService mainOrderService;

    @Autowired
    private IOrderRefundService orderRefundService;

    @Autowired
    @Qualifier("wxMiniPayService")
    private WxPayService wxMiniPayService;

    @Autowired
    @Qualifier("wxMiniPayRefundService")
    private  WxPayService wxMiniPayRefundService;

    @Autowired
    private RedissonClient redissonClient;


    /**
     * 支付超时或取消
     */
    public void PayCancel(MainOrders order) {
        //查询订单
        //查询小订单
        //修改大小订单状态
        //修改志愿者排班与预约时间表
        //修改平台流水表
    }


    /**
     * 订单支付回调
     */
    public String payNotify(String xmlData) {
        RLock fairLock = null;
        try {
            log.info("=====================>>>微信支付回调接收<<====================={}", xmlData);
            //解析微信回调数据
            WxPayOrderNotifyResult notifyResult = wxMiniPayService.parseOrderNotifyResult(xmlData);
            log.info("=====================>>>微信支付回调解析<<====================={}", notifyResult);
            //查询订单相关信息
            fairLock = redissonClient.getLock("order:notify:"+notifyResult.getOutTradeNo());
            //获取锁
            boolean lock = fairLock.tryLock(1000, TimeUnit.MILLISECONDS);
            if (lock){
                //再执行订单状态修改
                MainOrders mainOrders = mainOrderService.getOne(new QueryWrapper<MainOrders>().eq("main_order_id", notifyResult.getOutTradeNo()));
                if (ObjectUtil.isNull(mainOrders)){
                    log.error("微信支付回调结果异常1,异常原因：订单号无法找到对应订单{}",notifyResult.getOutTradeNo() );
                }
                //判断支付结果


                //向志愿者推送消息，并且插入未读消息
            }
            //根据商户单号查询订单，然后修改订单状态
            return WxPayNotifyResponse.success("成功");
        }catch (Exception e){
            log.error("微信支付回调结果异常2,异常原因：{}", e.getMessage());
            return WxPayNotifyResponse.fail("code:9999微信支付回调结果异常,异常原因:" + e.getMessage());
        }finally {
            if (ObjectUtil.isNotNull(fairLock) && fairLock.isLocked() && fairLock.isHeldByCurrentThread()) {
                fairLock.unlock();
            }
        }
    }


}
